
/*    
    This code is under GPL. Please read license terms and conditions at http://www.gnu.org/

    Yann LANGLAIS

    Changelog:
    12/03/2002  Creation
*/   

#include <stdio.h>
#include <strings.h>
#include <string.h>
#include <mem.h>

/* */
/*
	[A-z0-9_];
	==> 26 * 2 + 10 + 1 = 63;
	
	0    |    1    |    2    |    3    |    4    |    5    |    6    |    7
	01234567890123456789012345678901234567890123456789012345678901234567890
	0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz_
              ^                         ^                         ^
*/

/***********
0       0 NUL   1 SOH    2 STX    3 ETX    4 EOT    5 ENQ    6 ACK    7 BEL
1       8 BS    9 HT    10 NL    11 VT    12 NP    13 CR    14 SO    15 SI
2      16 DLE  17 DC1   18 DC2   19 DC3   20 DC4   21 NAK   22 SYN   23 ETB
3      24 CAN  25 EM    26 SUB   27 ESC   28 FS    29 GS    30 RS    31 US
4      32 SP   33 !     34 "     35 #     36 $     37 %     38 &     39 '
5      40 (    41 )     42 *     43 +     44 ,     45 -     46 .     47 /
6      48 0    49 1     50 2     51 3     52 4     53 5     54 6     55 7
7      56 8    57 9     58 :     59 ;     60 <     61 =     62 >     63 ?
8      64 @    65 A     66 B     67 C     68 D     69 E     70 F     71 G
9      72 H    73 I     74 J     75 K     76 L     77 M     78 N     79 O
A      80 P    81 Q     82 R     83 S     84 T     85 U     86 V     87 W
B      88 X    89 Y     90 Z     91 [     92 \     93 ]     94 ^     95 _
C      96 `    97 a     98 b     99 c    100 d    101 e    102 f    103 g
D     104 h   105 i    106 j    107 k    108 l    109 m    110 n    111 o
E     112 p   113 q    114 r    115 s    116 t    117 u    118 v    119 w
F     120 x   121 y    122 z    123 {    124 |    125 }    126 ~    127 DEL
***********/

static char __atr2__tt__[] =  {
    /*  0   1   2   3   4   5   6   7    */
/*0*/  63, 63, 63, 63, 63, 63, 63, 63,
/*1*/  63, 63, 63, 63, 63, 63, 63, 63,
/*2*/  63, 63, 63, 63, 63, 63, 63, 63,
/*3*/  63, 63, 63, 63, 63, 63, 63, 63,
/*4*/  63, 63, 63, 63, 63, 63, 63, 63,
/*5*/  63, 63, 63, 63, 63, 63, 63, 63,
/*6*/   0,  1,  2,  3,  4,  5,  6,  7,
/*7*/   8,  9, 63, 63, 63, 63, 63, 63,
/*8*/  63, 10, 11, 12, 13, 14, 15, 16,
/*9*/  17, 18, 19, 20, 21, 22, 23, 24,
/*A*/  25, 26, 27, 28, 29, 30, 31, 32,
/*B*/  33, 34, 35, 63, 63, 63, 63, 62, /** <--- '_' = 62!!! **/
/*C*/  63, 36, 37, 38, 39, 40, 41, 42,
/*D*/  43, 44, 45, 46, 47, 48, 49, 50,
/*E*/  51, 52, 53, 54, 55, 56, 57, 58,
/*F*/  59, 60, 61, 63, 63, 63, 63, 63
};

inline static int
atr2_char2index(register char c) {
	return __atr2__tt__[c];
}

typedef struct _atr2_t {
	struct _atr2_t *a[64];
	void          *data;
} atr2_t, *patr2_t;

patr2_t 
atr2_new() {
	return (patr2_t) mem_zmalloc(sizeof(atr2_t));
}

patr2_t 
atr2_destroy(patr2_t p) {
	register int i;
	if (!p) return NULL;
	/* destroy subtree */
	for (i = 0; i < 64; i++) atr2_destroy(p->a[i]);
	/* free memory */
	mem_free(p);
	return NULL;
}

void *
atr2_retrieve(register patr2_t p, register char *key) {
	register int i;	
	if (!p || !key) return NULL;
	while (*key && p) {
		i = atr2_char2index(*key);
		if (i >= 0 && i < 63) p = p->a[i];
		key++;
	}
	if (!*key && p) return p->data;
	return NULL;
}

void *
atr2_store(register patr2_t p, register char *key, register void *data) {
	register int i;
	if (!p | !key | !data) return NULL;
	while (*key) {
		i = atr2_char2index(*key);
		if (i >= 0 && i < 63) {
			if (!p->a[i] && !(p->a[i] = (patr2_t) mem_zmalloc(sizeof(atr2_t)))) return NULL;
			p = p->a[i];
		}
		key++;
	}
	return p->data = data;
}

void 
_atr2_dump_(patr2_t p, void (*f_dumpdata)(void *), int level, char *string) {
	int i;
	if (p) { 
		if (p->data) { 
			string[level] = 0;
			printf("%4d: %-20s --> ", level, string); 
			f_dumpdata(p->data); printf("\n");
		}
		for (i = 0; i < 64; i++) {
			if (p->a[i]) { 
				string[level] = i;
				_atr2_dump_(p->a[i], f_dumpdata, level + 1, string);
			}
		}
	}
}

void
atr2_dump(patr2_t p, void (*f_dumpdata)(void *)) {
	char str[1024];
	_atr2_dump_(p, f_dumpdata, 0, str);
}
